package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.HttpStatusEnum;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.mybatis.mapper.UserCardMapper;
import com.zanxiang.sdk.domain.params.UserCardUpdateParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.service.UserCardService;
import com.zanxiang.sdk.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * @author : lingfeng
 * @time : 2022-09-26
 * @description : 用户实名认证
 */
@Slf4j
@Service
public class UserCardServiceImpl extends ServiceImpl<UserCardMapper, UserCard> implements UserCardService {

    @Autowired
    private UserService userService;

    /**
     * 用户实名认证
     *
     * @param param    : 实名认证参数
     * @param userData : 用户登录信息
     * @return : 返回结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVO<Boolean> userAuthentication(UserCardUpdateParam param, UserData userData) {
        String cardId = param.getCardId();
        String cardName = param.getCardName();
        //todo : 调用阿里的实名认证接口
        int authentication = 1;
        UserCard userCard = super.getOne(new LambdaQueryWrapper<UserCard>()
                .eq(UserCard::getUserId, userData.getUserId()));
        if (userCard != null) {
            this.update(new LambdaUpdateWrapper<UserCard>()
                    .set(UserCard::getCardId, cardId)
                    .set(UserCard::getCardName, cardName)
                    .set(UserCard::getUpdateTime, LocalDateTime.now()));
            return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
        }
        //新建记录
        userCard = UserCard.builder()
                .userId(userData.getUserId())
                .cardId(cardId)
                .cardName(cardName)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
        this.save(userCard);
        //更新用户信息
        userService.update(new LambdaUpdateWrapper<User>()
                .set(User::getAuthentication, authentication)
                .eq(User::getId, userData.getUserId()));
        return new ResultVO<>(HttpStatusEnum.SUCCESS, Boolean.TRUE);
    }
}
